@@ -26,10 +26,12 @@ TEST_LOOP(trials, to_well_formed_utf16le_single_surrogate) {
   std::vector<char16_t> surrogates = {0xD800, 0xDC00, 0xDFFF, 0xD800, 0xDC00};
 #endif
   std::vector<char16_t> output(length);
+  for (size_t j = 0; j < length; j++)
+    utf16[j] = 0;
   for (size_t j = 0; j < length; j++) {
     for (char16_t surrogate : surrogates) {
       utf16[j] = surrogate;
-      size_t utf8_length =
+      simdutf::result utf8_length =
           implementation.utf8_length_from_utf16le_with_replacement(
               (const char16_t *)utf16.data(), utf16.size());
       std::fill(output.begin(), output.end(), 0);
@@ -38,7 +40,7 @@ TEST_LOOP(trials, to_well_formed_utf16le_single_surrogate) {
       size_t utf8_length_check = implementation.utf8_length_from_utf16le(
           (const char16_t *)output.data(), utf16.size());
       ASSERT_EQUAL(output[j], replacement_le);
-      ASSERT_EQUAL(utf8_length, utf8_length_check);
+      ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 
       utf16[j] = 0x0000; // Reset to a valid character
     }
@@ -57,7 +59,7 @@ TEST_LOOP(trials, to_well_formed_utf16be_single_surrogate) {
   for (size_t j = 0; j < length; j++) {
     for (char16_t surrogate : surrogates) {
       utf16[j] = surrogate;
-      size_t utf8_length =
+      simdutf::result utf8_length =
           implementation.utf8_length_from_utf16be_with_replacement(
               (const char16_t *)utf16.data(), utf16.size());
       std::fill(output.begin(), output.end(), 0);
@@ -66,7 +68,8 @@ TEST_LOOP(trials, to_well_formed_utf16be_single_surrogate) {
       size_t utf8_length_check = implementation.utf8_length_from_utf16be(
           (const char16_t *)output.data(), utf16.size());
       ASSERT_EQUAL(output[j], replacement_be);
-      ASSERT_EQUAL(utf8_length, utf8_length_check);
+      ASSERT_EQUAL(utf8_length.count, utf8_length_check);
+      ASSERT_EQUAL(utf8_length.error, simdutf::SURROGATE);
       utf16[j] = 0x0000; // Reset to a valid character
     }
   }
@@ -79,13 +82,14 @@ TEST_LOOP(trials,
   const auto utf16{generator.generate_le(8)};
   const auto len = utf16.size();
   std::vector<char16_t> output(len);
-  size_t utf8_length = implementation.utf8_length_from_utf16le_with_replacement(
-      utf16.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16le_with_replacement(utf16.data(),
+                                                               len);
   implementation.to_well_formed_utf16le(utf16.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16le(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 TEST_LOOP(trials,
@@ -94,13 +98,14 @@ TEST_LOOP(trials,
   const auto utf16{generator.generate_be(8)};
   const auto len = utf16.size();
   std::vector<char16_t> output(len);
-  size_t utf8_length = implementation.utf8_length_from_utf16be_with_replacement(
-      utf16.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16be_with_replacement(utf16.data(),
+                                                               len);
   implementation.to_well_formed_utf16be(utf16.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16be(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 TEST_LOOP(trials, to_well_formed_utf16le_for_valid_input_surrogate_pairs_long) {
@@ -108,13 +113,14 @@ TEST_LOOP(trials, to_well_formed_utf16le_for_valid_input_surrogate_pairs_long) {
   const auto utf16{generator.generate_le(512)};
   const auto len = utf16.size();
   std::vector<char16_t> output(len);
-  size_t utf8_length = implementation.utf8_length_from_utf16le_with_replacement(
-      utf16.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16le_with_replacement(utf16.data(),
+                                                               len);
   implementation.to_well_formed_utf16le(utf16.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16le(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 TEST_LOOP(trials, to_well_formed_utf16be_for_valid_input_surrogate_pairs_long) {
@@ -122,13 +128,14 @@ TEST_LOOP(trials, to_well_formed_utf16be_for_valid_input_surrogate_pairs_long) {
   const auto utf16{generator.generate_be(512)};
   const auto len = utf16.size();
   std::vector<char16_t> output(len);
-  size_t utf8_length = implementation.utf8_length_from_utf16be_with_replacement(
-      utf16.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16be_with_replacement(utf16.data(),
+                                                               len);
   implementation.to_well_formed_utf16be(utf16.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16be(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 TEST_LOOP(trials, to_well_formed_utf16le_for_valid_input_mixed_long) {
@@ -136,13 +143,14 @@ TEST_LOOP(trials, to_well_formed_utf16le_for_valid_input_mixed_long) {
   const auto utf16{generator.generate_le(512)};
   const auto len = utf16.size();
   std::vector<char16_t> output(len);
-  size_t utf8_length = implementation.utf8_length_from_utf16le_with_replacement(
-      utf16.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16le_with_replacement(utf16.data(),
+                                                               len);
   implementation.to_well_formed_utf16le(utf16.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16le(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 TEST_LOOP(trials, to_well_formed_utf16be_for_valid_input_mixed_long) {
@@ -150,13 +158,14 @@ TEST_LOOP(trials, to_well_formed_utf16be_for_valid_input_mixed_long) {
   const auto utf16{generator.generate_be(512)};
   const auto len = utf16.size();
   std::vector<char16_t> output(len);
-  size_t utf8_length = implementation.utf8_length_from_utf16be_with_replacement(
-      utf16.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16be_with_replacement(utf16.data(),
+                                                               len);
   implementation.to_well_formed_utf16be(utf16.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16be(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 TEST_LOOP(trials, to_well_formed_utf16le_for_valid_input_mixed_long_self) {
@@ -164,13 +173,14 @@ TEST_LOOP(trials, to_well_formed_utf16le_for_valid_input_mixed_long_self) {
   const auto utf16{generator.generate_le(512)};
   const auto len = utf16.size();
   std::vector<char16_t> output = utf16;
-  size_t utf8_length = implementation.utf8_length_from_utf16le_with_replacement(
-      output.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16le_with_replacement(output.data(),
+                                                               len);
   implementation.to_well_formed_utf16le(output.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16le(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 TEST_LOOP(trials, to_well_formed_utf16be_for_valid_input_mixed_long_self) {
@@ -178,13 +188,14 @@ TEST_LOOP(trials, to_well_formed_utf16be_for_valid_input_mixed_long_self) {
   const auto utf16{generator.generate_be(512)};
   const auto len = utf16.size();
   std::vector<char16_t> output = utf16;
-  size_t utf8_length = implementation.utf8_length_from_utf16be_with_replacement(
-      output.data(), len);
+  simdutf::result utf8_length =
+      implementation.utf8_length_from_utf16be_with_replacement(output.data(),
+                                                               len);
   implementation.to_well_formed_utf16be(output.data(), len, output.data());
   size_t utf8_length_check =
       implementation.utf8_length_from_utf16be(output.data(), len);
   ASSERT_EQUAL(output, utf16);
-  ASSERT_EQUAL(utf8_length, utf8_length_check);
+  ASSERT_EQUAL(utf8_length.count, utf8_length_check);
 }
 
 std::vector<char16_t> random_testcase(size_t n, std::mt19937 &rng) {
@@ -213,19 +224,20 @@ TEST(to_well_formed_utf16le_bad_input) {
     auto utf16 = random_testcase(512, gen);
     auto len = utf16.size();
     std::vector<char16_t> output(len);
-    size_t utf8_length =
+    simdutf::result utf8_length =
         implementation.utf8_length_from_utf16le_with_replacement(utf16.data(),
                                                                  len);
     implementation.to_well_formed_utf16le(utf16.data(), len, output.data());
     for (size_t j = 0; j < len; j++) {
       if (utf16[j] != output[j]) {
         ASSERT_EQUAL(output[j], replacement_le);
+        ASSERT_EQUAL(utf8_length.error, simdutf::SURROGATE);
       }
     }
     size_t utf8_length_check =
         implementation.utf8_length_from_utf16le(output.data(), len);
     ASSERT_TRUE(implementation.validate_utf16le(output.data(), len));
-    ASSERT_EQUAL(utf8_length, utf8_length_check);
+    ASSERT_EQUAL(utf8_length.count, utf8_length_check);
   }
 }
 
@@ -235,19 +247,20 @@ TEST(to_well_formed_utf16be_bad_input) {
     auto utf16 = random_testcase(512, gen);
     auto len = utf16.size();
     std::vector<char16_t> output(len);
-    size_t utf8_length =
+    simdutf::result utf8_length =
         implementation.utf8_length_from_utf16be_with_replacement(utf16.data(),
                                                                  len);
     implementation.to_well_formed_utf16be(utf16.data(), len, output.data());
     for (size_t j = 0; j < len; j++) {
       if (utf16[j] != output[j]) {
         ASSERT_EQUAL(output[j], replacement_be);
+        ASSERT_EQUAL(utf8_length.error, simdutf::SURROGATE);
       }
     }
     size_t utf8_length_check =
         implementation.utf8_length_from_utf16be(output.data(), len);
     ASSERT_TRUE(implementation.validate_utf16be(output.data(), len));
-    ASSERT_EQUAL(utf8_length, utf8_length_check);
+    ASSERT_EQUAL(utf8_length.count, utf8_length_check);
   }
 }
 
@@ -257,19 +270,20 @@ TEST(to_well_formed_utf16le_bad_input_self) {
     auto utf16 = random_testcase(512, gen);
     auto len = utf16.size();
     std::vector<char16_t> output = utf16;
-    size_t utf8_length =
+    simdutf::result utf8_length =
         implementation.utf8_length_from_utf16le_with_replacement(output.data(),
                                                                  len);
     implementation.to_well_formed_utf16le(output.data(), len, output.data());
     for (size_t j = 0; j < len; j++) {
       if (utf16[j] != output[j]) {
         ASSERT_EQUAL(output[j], replacement_le);
+        ASSERT_EQUAL(utf8_length.error, simdutf::SURROGATE);
       }
     }
     size_t utf8_length_check =
         implementation.utf8_length_from_utf16le(output.data(), len);
     ASSERT_TRUE(implementation.validate_utf16le(output.data(), len));
-    ASSERT_EQUAL(utf8_length, utf8_length_check);
+    ASSERT_EQUAL(utf8_length.count, utf8_length_check);
   }
 }
 
@@ -279,19 +293,20 @@ TEST(to_well_formed_utf16be_bad_input_self) {
     auto utf16 = random_testcase(512, gen);
     auto len = utf16.size();
     std::vector<char16_t> output = utf16;
-    size_t utf8_length =
+    simdutf::result utf8_length =
         implementation.utf8_length_from_utf16be_with_replacement(output.data(),
                                                                  len);
     implementation.to_well_formed_utf16be(output.data(), len, output.data());
     for (size_t j = 0; j < len; j++) {
       if (utf16[j] != output[j]) {
         ASSERT_EQUAL(output[j], replacement_be);
+        ASSERT_EQUAL(utf8_length.error, simdutf::SURROGATE);
       }
     }
     size_t utf8_length_check =
         implementation.utf8_length_from_utf16be(output.data(), len);
     ASSERT_TRUE(implementation.validate_utf16be(output.data(), len));
-    ASSERT_EQUAL(utf8_length, utf8_length_check);
+    ASSERT_EQUAL(utf8_length.count, utf8_length_check);
   }
 }
 
